@@ -30,6 +30,7 @@
 
 
 #include "draw/draw_context.h"
+#include "util/u_helpers.h"
 #include "util/u_inlines.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
@@ -177,7 +178,9 @@ static void i915_bind_blend_state(struct pipe_context *pipe,
                                   void *blend)
 {
    struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
+
+   if (i915->blend == blend)
+      return;
 
    i915->blend = (struct i915_blend_state*)blend;
 
@@ -194,7 +197,9 @@ static void i915_set_blend_color( struct pipe_context *pipe,
                                   const struct pipe_blend_color *blend_color )
 {
    struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
+
+   if (!blend_color)
+      return;
 
    i915->blend_color = *blend_color;
 
@@ -205,7 +210,6 @@ static void i915_set_stencil_ref( struct pipe_context *pipe,
                                   const struct pipe_stencil_ref *stencil_ref )
 {
    struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
 
    i915->stencil_ref = *stencil_ref;
 
@@ -288,17 +292,6 @@ i915_create_sampler_state(struct pipe_context *pipe,
    return cso;
 }
 
-static void i915_fixup_bind_sampler_states(struct pipe_context *pipe,
-                                           unsigned num, void **sampler)
-{
-   struct i915_context *i915 = i915_context(pipe);
-
-   i915->saved_nr_samplers = num;
-   memcpy(&i915->saved_samplers, sampler, sizeof(void *) * num);
-
-   i915->saved_bind_sampler_states(pipe, num, sampler);
-}
-
 static void
 i915_bind_vertex_sampler_states(struct pipe_context *pipe,
                                 unsigned num_samplers,
@@ -307,23 +300,22 @@ i915_bind_vertex_sampler_states(struct pipe_context *pipe,
    struct i915_context *i915 = i915_context(pipe);
    unsigned i;
 
-   assert(num_samplers <= PIPE_MAX_VERTEX_SAMPLERS);
+   assert(num_samplers <= Elements(i915->vertex_samplers));
 
    /* Check for no-op */
    if (num_samplers == i915->num_vertex_samplers &&
        !memcmp(i915->vertex_samplers, samplers, num_samplers * sizeof(void *)))
       return;
 
-   draw_flush(i915->draw);
-
    for (i = 0; i < num_samplers; ++i)
       i915->vertex_samplers[i] = samplers[i];
-   for (i = num_samplers; i < PIPE_MAX_VERTEX_SAMPLERS; ++i)
+   for (i = num_samplers; i < Elements(i915->vertex_samplers); ++i)
       i915->vertex_samplers[i] = NULL;
 
    i915->num_vertex_samplers = num_samplers;
 
    draw_set_samplers(i915->draw,
+                     PIPE_SHADER_VERTEX,
                      i915->vertex_samplers,
                      i915->num_vertex_samplers);
 }
@@ -341,8 +333,6 @@ static void i915_bind_fragment_sampler_states(struct pipe_context *pipe,
        !memcmp(i915->sampler, sampler, num * sizeof(void *)))
       return;
 
-   draw_flush(i915->draw);
-
    for (i = 0; i < num; ++i)
       i915->sampler[i] = sampler[i];
    for (i = num; i < PIPE_MAX_SAMPLERS; ++i)
@@ -370,15 +360,15 @@ i915_prepare_vertex_sampling(struct i915_context *i915)
    unsigned i,j;
    uint32_t row_stride[PIPE_MAX_TEXTURE_LEVELS];
    uint32_t img_stride[PIPE_MAX_TEXTURE_LEVELS];
-   const void* data[PIPE_MAX_TEXTURE_LEVELS];
+   uint32_t mip_offsets[PIPE_MAX_TEXTURE_LEVELS];
    unsigned num = i915->num_vertex_sampler_views;
    struct pipe_sampler_view **views = i915->vertex_sampler_views;
 
-   assert(num <= PIPE_MAX_VERTEX_SAMPLERS);
+   assert(num <= PIPE_MAX_SAMPLERS);
    if (!num)
       return;
 
-   for (i = 0; i < PIPE_MAX_VERTEX_SAMPLERS; i++) {
+   for (i = 0; i < PIPE_MAX_SAMPLERS; i++) {
       struct pipe_sampler_view *view = i < num ? views[i] : NULL;
 
       if (view) {
@@ -391,25 +381,26 @@ i915_prepare_vertex_sampling(struct i915_context *i915)
           */
          pipe_resource_reference(&i915->mapped_vs_tex[i], tex);
 
-	 i915->mapped_vs_tex_buffer[i] = i915_tex->buffer;
+         i915->mapped_vs_tex_buffer[i] = i915_tex->buffer;
          addr = iws->buffer_map(iws,
                                 i915_tex->buffer,
                                 FALSE /* read only */);
 
          /* Setup array of mipmap level pointers */
-	 /* FIXME: handle 3D textures? */
+         /* FIXME: handle 3D textures? */
          for (j = view->u.tex.first_level; j <= tex->last_level; j++) {
-            unsigned offset = i915_texture_offset(i915_tex, j , 0 /* FIXME depth */);
-            data[j] = addr + offset;
+            mip_offsets[j] = i915_texture_offset(i915_tex, j , 0 /* FIXME depth */);
             row_stride[j] = i915_tex->stride;
             img_stride[j] = 0; /* FIXME */;
          }
 
          draw_set_mapped_texture(i915->draw,
+                                 PIPE_SHADER_VERTEX,
                                  i,
                                  tex->width0, tex->height0, tex->depth0,
                                  view->u.tex.first_level, tex->last_level,
-                                 row_stride, img_stride, data);
+                                 addr,
+                                 row_stride, img_stride, mip_offsets);
       } else
          i915->mapped_vs_tex[i] = NULL;
    }
@@ -527,7 +518,9 @@ static void i915_bind_depth_stencil_state(struct pipe_context *pipe,
                                           void *depth_stencil)
 {
    struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
+
+   if (i915->depth_stencil == depth_stencil)
+      return;
 
    i915->depth_stencil = (const struct i915_depth_stencil_state *)depth_stencil;
 
@@ -545,7 +538,6 @@ static void i915_set_scissor_state( struct pipe_context *pipe,
                                  const struct pipe_scissor_state *scissor )
 {
    struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
 
    memcpy( &i915->scissor, scissor, sizeof(*scissor) );
    i915->dirty |= I915_NEW_SCISSOR;
@@ -579,22 +571,13 @@ i915_create_fs_state(struct pipe_context *pipe,
    return ifs;
 }
 
-static void
-i915_fixup_bind_fs_state(struct pipe_context *pipe, void *shader)
-{
-   struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
-
-   i915->saved_fs = shader;
-
-   i915->saved_bind_fs_state(pipe, shader);
-}
-
 static void
 i915_bind_fs_state(struct pipe_context *pipe, void *shader)
 {
    struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
+
+   if (i915->fs == shader)
+      return;
 
    i915->fs = (struct i915_fragment_shader*) shader;
 
@@ -608,13 +591,18 @@ void i915_delete_fs_state(struct pipe_context *pipe, void *shader)
 {
    struct i915_fragment_shader *ifs = (struct i915_fragment_shader *) shader;
 
+   FREE(ifs->decl);
+   ifs->decl = NULL;
+
    if (ifs->program) {
       FREE(ifs->program);
       ifs->program = NULL;
       FREE((struct tgsi_token *)ifs->state.tokens);
       ifs->state.tokens = NULL;
    }
+
    ifs->program_len = 0;
+   ifs->decl_len = 0;
 
    FREE(ifs);
 }
@@ -634,7 +622,10 @@ static void i915_bind_vs_state(struct pipe_context *pipe, void *shader)
 {
    struct i915_context *i915 = i915_context(pipe);
 
-   i915->saved_vs = shader;
+   if (i915->vs == shader)
+      return;
+
+   i915->vs = shader;
 
    /* just pass-through to draw module */
    draw_bind_vertex_shader(i915->draw, (struct draw_vertex_shader *) shader);
@@ -652,17 +643,23 @@ static void i915_delete_vs_state(struct pipe_context *pipe, void *shader)
 
 static void i915_set_constant_buffer(struct pipe_context *pipe,
                                      uint shader, uint index,
-                                     struct pipe_resource *buf)
+                                     struct pipe_constant_buffer *cb)
 {
    struct i915_context *i915 = i915_context(pipe);
+   struct pipe_resource *buf = cb ? cb->buffer : NULL;
    unsigned new_num = 0;
    boolean diff = TRUE;
 
-
    /* XXX don't support geom shaders now */
    if (shader == PIPE_SHADER_GEOMETRY)
       return;
 
+   if (cb && cb->user_buffer) {
+      buf = i915_user_buffer_create(pipe->screen, (void *) cb->user_buffer,
+                                    cb->buffer_size,
+                                    PIPE_BIND_CONSTANT_BUFFER);
+   }
+
    /* if we have a new buffer compare it with the old one */
    if (buf) {
       struct i915_buffer *ibuf = i915_buffer(buf);
@@ -688,41 +685,18 @@ static void i915_set_constant_buffer(struct pipe_context *pipe,
       diff = i915->current.num_user_constants[shader] != 0;
    }
 
-   /*
-    * flush before updateing the state.
-    */
-   if (diff && shader == PIPE_SHADER_FRAGMENT)
-      draw_flush(i915->draw);
-
    pipe_resource_reference(&i915->constants[shader], buf);
    i915->current.num_user_constants[shader] = new_num;
 
    if (diff)
       i915->dirty |= shader == PIPE_SHADER_VERTEX ? I915_NEW_VS_CONSTANTS : I915_NEW_FS_CONSTANTS;
+
+   if (cb && cb->user_buffer) {
+      pipe_resource_reference(&buf, NULL);
+   }
 }
 
 
-static void
-i915_fixup_set_fragment_sampler_views(struct pipe_context *pipe,
-                                      unsigned num,
-                                      struct pipe_sampler_view **views)
-{
-   struct i915_context *i915 = i915_context(pipe);
-   int i;
-
-   for (i = 0; i < num; i++)
-      pipe_sampler_view_reference(&i915->saved_sampler_views[i],
-                                  views[i]);
-
-   for (i = num; i < i915->saved_nr_sampler_views; i++)
-      pipe_sampler_view_reference(&i915->saved_sampler_views[i],
-                                  NULL);
-
-   i915->saved_nr_sampler_views = num;
-
-   i915->saved_set_sampler_views(pipe, num, views);
-}
-
 static void i915_set_fragment_sampler_views(struct pipe_context *pipe,
                                             unsigned num,
                                             struct pipe_sampler_view **views)
@@ -737,9 +711,6 @@ static void i915_set_fragment_sampler_views(struct pipe_context *pipe,
        !memcmp(i915->fragment_sampler_views, views, num * sizeof(struct pipe_sampler_view *)))
       return;
 
-   /* Fixes wrong texture in texobj with VBUF */
-   draw_flush(i915->draw);
-
    for (i = 0; i < num; i++)
       pipe_sampler_view_reference(&i915->fragment_sampler_views[i],
                                   views[i]);
@@ -761,7 +732,7 @@ i915_set_vertex_sampler_views(struct pipe_context *pipe,
    struct i915_context *i915 = i915_context(pipe);
    uint i;
 
-   assert(num <= PIPE_MAX_VERTEX_SAMPLERS);
+   assert(num <= Elements(i915->vertex_sampler_views));
 
    /* Check for no-op */
    if (num == i915->num_vertex_sampler_views &&
@@ -769,9 +740,7 @@ i915_set_vertex_sampler_views(struct pipe_context *pipe,
       return;
    }
 
-   draw_flush(i915->draw);
-
-   for (i = 0; i < PIPE_MAX_VERTEX_SAMPLERS; i++) {
+   for (i = 0; i < Elements(i915->vertex_sampler_views); i++) {
       struct pipe_sampler_view *view = i < num ? views[i] : NULL;
 
       pipe_sampler_view_reference(&i915->vertex_sampler_views[i], view);
@@ -780,6 +749,7 @@ i915_set_vertex_sampler_views(struct pipe_context *pipe,
    i915->num_vertex_sampler_views = num;
 
    draw_set_sampler_views(i915->draw,
+                          PIPE_SHADER_VERTEX,
                           i915->vertex_sampler_views,
                           i915->num_vertex_sampler_views);
 }
@@ -819,8 +789,6 @@ static void i915_set_framebuffer_state(struct pipe_context *pipe,
    struct i915_context *i915 = i915_context(pipe);
    int i;
 
-   draw_flush(i915->draw);
-
    i915->framebuffer.width = fb->width;
    i915->framebuffer.height = fb->height;
    i915->framebuffer.nr_cbufs = fb->nr_cbufs;
@@ -839,9 +807,8 @@ static void i915_set_clip_state( struct pipe_context *pipe,
 			     const struct pipe_clip_state *clip )
 {
    struct i915_context *i915 = i915_context(pipe);
-   draw_flush(i915->draw);
 
-   i915->saved_clip = *clip;
+   i915->clip = *clip;
 
    draw_set_clip_state(i915->draw, clip);
 
@@ -940,6 +907,9 @@ static void i915_bind_rasterizer_state( struct pipe_context *pipe,
 {
    struct i915_context *i915 = i915_context(pipe);
 
+   if (i915->rasterizer == raster)
+      return;
+
    i915->rasterizer = (struct i915_rasterizer_state *)raster;
 
    /* pass-through to draw module */
@@ -957,32 +927,18 @@ static void i915_delete_rasterizer_state(struct pipe_context *pipe,
 }
 
 static void i915_set_vertex_buffers(struct pipe_context *pipe,
-                                    unsigned count,
+                                    unsigned start_slot, unsigned count,
                                     const struct pipe_vertex_buffer *buffers)
 {
    struct i915_context *i915 = i915_context(pipe);
    struct draw_context *draw = i915->draw;
-   int i;
 
-   util_copy_vertex_buffers(i915->saved_vertex_buffers,
-                            &i915->saved_nr_vertex_buffers,
-                            buffers, count);
-#if 0
-   /* XXX doesn't look like this is needed */
-   /* unmap old */
-   for (i = 0; i < i915->num_vertex_buffers; i++) {
-      draw_set_mapped_vertex_buffer(draw, i, NULL);
-   }
-#endif
+   util_set_vertex_buffers_count(i915->vertex_buffers,
+                                 &i915->nr_vertex_buffers,
+                                 buffers, start_slot, count);
 
    /* pass-through to draw module */
-   draw_set_vertex_buffers(draw, count, buffers);
-
-   /* map new */
-   for (i = 0; i < count; i++) {
-      void *buf = i915_buffer(buffers[i].buffer)->data;
-      draw_set_mapped_vertex_buffer(draw, i, buf);
-   }
+   draw_set_vertex_buffers(draw, start_slot, count, buffers);
 }
 
 static void *
@@ -1007,7 +963,10 @@ i915_bind_vertex_elements_state(struct pipe_context *pipe,
    struct i915_context *i915 = i915_context(pipe);
    struct i915_velems_state *i915_velems = (struct i915_velems_state *) velems;
 
-   i915->saved_velems = velems;
+   if (i915->velems == velems)
+      return;
+
+   i915->velems = velems;
 
    /* pass-through to draw module */
    if (i915_velems) {
@@ -1031,9 +990,6 @@ static void i915_set_index_buffer(struct pipe_context *pipe,
       memcpy(&i915->index_buffer, ib, sizeof(i915->index_buffer));
    else
       memset(&i915->index_buffer, 0, sizeof(i915->index_buffer));
-
-   /* pass-through to draw module */
-   draw_set_index_buffer(i915->draw, ib);
 }
 
 static void
@@ -1087,16 +1043,4 @@ i915_init_state_functions( struct i915_context *i915 )
    i915->base.set_viewport_state = i915_set_viewport_state;
    i915->base.set_vertex_buffers = i915_set_vertex_buffers;
    i915->base.set_index_buffer = i915_set_index_buffer;
-   i915->base.redefine_user_buffer = u_default_redefine_user_buffer;
-}
-
-void
-i915_init_fixup_state_functions( struct i915_context *i915 )
-{
-   i915->saved_bind_fs_state = i915->base.bind_fs_state;
-   i915->base.bind_fs_state = i915_fixup_bind_fs_state;
-   i915->saved_bind_sampler_states = i915->base.bind_fragment_sampler_states;
-   i915->base.bind_fragment_sampler_states = i915_fixup_bind_sampler_states;
-   i915->saved_set_sampler_views = i915->base.set_fragment_sampler_views;
-   i915->base.set_fragment_sampler_views = i915_fixup_set_fragment_sampler_views;
 }
